function [de_scores, de_pvalues] = app2(kernel_exp, de_exp, all_exp, de_genes)
%INPUTS:
%kernel_exp: expressions of kernel genes/probes over the specified conditions, a 
%numeric matrix s.t. each row corresponds to the expression vector of a kernel gene
%
%de_exp: expressions of differentially expressed genes/probes over the specified conditions, 
%a numeric matrix s.t. each row corresponds to the expression vector of a differntially expressed gene
%
%all_exp: expressions of all genes/probs in microarray over the specified conditions (for p-value
%calculation)
%
%de_genes: list of differentially expressed genes, gene in row_i has its
%expression vector in row_i of 'de_exp'
%
%OUTPUTS: 
%de_scores: KCS scores of differentially expressed genes
%p-values of the KSC scores

% cluster kernels
[clusters, centres] = clustering( kernel_exp, 0.8, 0.05 );
% remove singleton clusters
[~, updated_centre] = rmv_single( clusters, centres);
%calculate KSC scores
de_scores = kcs(de_exp, updated_centre, kernel_exp );

% randomly sample from all gene'probes expressions for the permutation test
n = size(all_exp,1);
sample_ind = randsample(n,10000,true);
sample_exp = all_exp(sample_ind,:);
sample_scores = kcs(sample_exp, updated_centre, kernel_exp);

% permutation test, calculate KSC p-values
de_pvalues = perm_test(de_scores, sample_scores);

fileID2 = fopen('app2_results.csv','w');
fprintf(fileID2,'Gene_Symbol,KCS,p-value\n');

for i = 1:numel(de_genes)
    fprintf(fileID2,'%s,%f,%f\n',de_genes{i},de_scores(i),de_pvalues(i));
end

fclose(fileID2);

end
